@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2007-2012 Red Hat, Inc.
+# Copyright (C) 2007-2016 Red Hat, Inc.
 # Authors:
 # Thomas Woerner <twoerner@redhat.com>
 #
@@ -18,6 +18,8 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
+from __future__ import absolute_import
+
 # translation
 import locale
 try:
@@ -31,14 +33,16 @@ DOMAIN = 'firewalld'
 import gettext
 gettext.install(domain=DOMAIN)
 
+from . import dbus
+
 # configuration
 DAEMON_NAME = 'firewalld'
 CONFIG_NAME = 'firewall-config'
 APPLET_NAME = 'firewall-applet'
 DATADIR = '/usr/share/' + DAEMON_NAME
 CONFIG_GLADE_NAME = CONFIG_NAME + '.glade'
-COPYRIGHT = '(C) 2010-2015 Red Hat, Inc.'
-VERSION = '0.4.0'
+COPYRIGHT = '(C) 2010-2016 Red Hat, Inc.'
+VERSION = '0.4.1.2'
 AUTHORS = [
     "Thomas Woerner <twoerner@redhat.com>",
     "Jiri Popelka <jpopelka@redhat.com>",
@@ -73,6 +77,8 @@ FIREWALLD_IPSETS = USR_LIB_FIREWALLD + '/ipsets'
 
 FIREWALLD_LOGFILE = '/var/log/firewalld'
 
+FIREWALLD_PIDFILE = "/var/run/firewalld.pid"
+
 FIREWALLD_TEMPDIR = '/run/firewalld'
 
 FIREWALLD_DIRECT = ETC_FIREWALLD + '/direct.xml'
