@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2010-2012 Red Hat, Inc.
+# Copyright (C) 2010-2016 Red Hat, Inc.
 #
 # Authors:
 # Thomas Woerner <twoerner@redhat.com>
@@ -19,6 +19,8 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
+__all__ = [ "ebtables" ]
+
 import os.path, errno
 from firewall.core.prog import runProg
 from firewall.core.logger import log
@@ -54,13 +56,14 @@ class ebtables(object):
         self._restore_command = COMMANDS["%s-restore" % self.ipv]
         self.ebtables_lock = "/var/lib/ebtables/lock"
         self.restore_noflush_option = self._detect_restore_noflush_option()
+        self.concurrent_option = self._detect_concurrent_option()
         self.__remove_dangling_lock()
 
     def __remove_dangling_lock(self):
         if os.path.exists(self.ebtables_lock):
-            (status, ret) = runProg("pidof", [ "-s", "ebtables" ])
-            (status2, ret2) = runProg("pidof", [ "-s", "ebtables-restore" ])
-            if ret == "" and ret2 == "":
+            ret = runProg("pidof", [ "-s", "ebtables" ])
+            ret2 = runProg("pidof", [ "-s", "ebtables-restore" ])
+            if ret[1] == "" and ret2[1] == "":
                 log.warning("Removing dangling ebtables lock file: '%s'" %
                             self.ebtables_lock)
                 try:
@@ -69,19 +72,32 @@ class ebtables(object):
                     if e.errno != errno.ENOENT:
                         raise
 
+    def _detect_concurrent_option(self):
+        # Do not change any rules, just try to use the --concurrent option
+        # with -L
+        concurrent_option = ""
+        (status, ret) = runProg(self._command, ["--concurrent", "-L"])
+        if status == 0:
+            concurrent_option = "--concurrent"  # concurrent for ebtables lock
+
+        return concurrent_option
+
     def _detect_restore_noflush_option(self):
         # Do not change any rules, just try to use the restore command
         # with --noflush
         rules = [ ]
         try:
             self.set_rules(rules, flush=False)
-        except ValueError as e:
+        except ValueError:
             return False
         return True
 
     def __run(self, args):
         # convert to string list
-        _args = ["--concurrent"] + ["%s" % item for item in args]
+        _args = [ ]
+        if self.concurrent_option and self.concurrent_option not in args:
+            _args.append(self.concurrent_option)
+        _args += ["%s" % item for item in args]
         log.debug2("%s: %s %s", self.__class__, self._command, " ".join(_args))
         self.__remove_dangling_lock()
         (status, ret) = runProg(self._command, _args)
@@ -98,7 +114,7 @@ class ebtables(object):
         for rule in rules:
             try:
                 i = rule.index("-t")
-            except:
+            except Exception:
                 pass
             else:
                 if len(rule) >= i+1:
@@ -127,7 +143,7 @@ class ebtables(object):
 
         if log.getDebugLogLevel() > 2:
             try:
-                lines = readfile(temp_file.name)
+                readfile(temp_file.name)
             except:
                 pass
             else:
@@ -206,10 +222,9 @@ class ebtables(object):
                     try:
                         self.__run([ "-t", table, "-P", chain, policy ])
                     except Exception as msg:
-                        log.error("Failed to set policy for %s: %s", ipv, msg)
+                        log.error("Failed to set policy for %s: %s", self.ipv,
+                                  msg)
                 else:
                     rules.append([ "-t", table, "-P", chain, policy ])
         if len(rules) > 0:
             self.set_rules(rules)
-
-ebtables_available_tables = ebtables().available_tables()
