@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2010-2012 Red Hat, Inc.
+# Copyright (C) 2010-2016 Red Hat, Inc.
 #
 # Authors:
 # Thomas Woerner <twoerner@redhat.com>
@@ -19,9 +19,11 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
-import os.path
+__all__ = [ "Firewall" ]
+
+import os.path, sys
 import copy
-from firewall.config import *
+from firewall import config
 from firewall import functions
 from firewall.core import ipXtables
 from firewall.core import ebtables
@@ -41,7 +43,8 @@ from firewall.core.io.service import service_reader
 from firewall.core.io.icmptype import icmptype_reader
 from firewall.core.io.zone import zone_reader, Zone
 from firewall.core.io.ipset import ipset_reader
-from firewall.errors import *
+from firewall import errors
+from firewall.errors import FirewallError
 
 ############################################################################
 #
@@ -51,18 +54,24 @@ from firewall.errors import *
 
 class Firewall(object):
     def __init__(self):
-        self._firewalld_conf = firewalld_conf(FIREWALLD_CONF)
+        self._firewalld_conf = firewalld_conf(config.FIREWALLD_CONF)
 
-        self._ip4tables = ipXtables.ip4tables()
+        self.ip4tables_backend = ipXtables.ip4tables()
         self.ip4tables_enabled = True
-        self._ip6tables = ipXtables.ip6tables()
+        self.ip6tables_backend = ipXtables.ip6tables()
         self.ip6tables_enabled = True
-        self._ebtables = ebtables.ebtables()
+        self.ebtables_backend = ebtables.ebtables()
         self.ebtables_enabled = True
-        self._ipset = ipset.ipset()
+        self.ipset_backend = ipset.ipset()
         self.ipset_enabled = True
+        self.ipset_supported_types = [ ]
 
-        self._modules = modules.modules()
+        self.available_tables = { }
+        self.available_tables["ipv4"] = self.ip4tables_backend.available_tables()
+        self.available_tables["ipv6"] = self.ip6tables_backend.available_tables()
+        self.available_tables["eb"] = self.ebtables_backend.available_tables()
+
+        self.modules_backend = modules.modules()
 
         self.icmptype = FirewallIcmpType(self)
         self.service = FirewallService(self)
@@ -75,7 +84,7 @@ class Firewall(object):
         self.__init_vars()
 
     def __repr__(self):
-        return '%s(%r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r %r)' % \
+        return '%s(%r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r)' % \
             (self.__class__, self.ip4tables_enabled, self.ip6tables_enabled,
              self.ebtables_enabled, self._state, self._panic,
              self._default_zone, self._module_refcount, self._marks,
@@ -89,46 +98,55 @@ class Firewall(object):
         self._module_refcount = { }
         self._marks = [ ]
         # fallback settings will be overloaded by firewalld.conf
-        self._min_mark = FALLBACK_MINIMAL_MARK
-        self.cleanup_on_exit = FALLBACK_CLEANUP_ON_EXIT
-        self.ipv6_rpfilter_enabled = FALLBACK_IPV6_RPFILTER
-        self._individual_calls = FALLBACK_INDIVIDUAL_CALLS
-        self._log_denied = FALLBACK_LOG_DENIED
+        self._min_mark = config.FALLBACK_MINIMAL_MARK
+        self.cleanup_on_exit = config.FALLBACK_CLEANUP_ON_EXIT
+        self.ipv6_rpfilter_enabled = config.FALLBACK_IPV6_RPFILTER
+        self._individual_calls = config.FALLBACK_INDIVIDUAL_CALLS
+        self._log_denied = config.FALLBACK_LOG_DENIED
+
+    def individual_calls(self):
+        return self._individual_calls
 
     def _check_tables(self):
         # check if iptables, ip6tables and ebtables are usable, else disable
         if self.ip4tables_enabled and \
-           "filter" not in ipXtables.ip4tables_available_tables:
+           "filter" not in self.get_available_tables("ipv4"):
             log.warning("iptables not usable, disabling IPv4 firewall.")
             self.ip4tables_enabled = False
 
         if self.ip6tables_enabled and \
-           "filter" not in ipXtables.ip6tables_available_tables:
+           "filter" not in self.get_available_tables("ipv6"):
             log.warning("ip6tables not usable, disabling IPv6 firewall.")
             self.ip6tables_enabled = False
 
         if self.ebtables_enabled and \
-           "filter" not in ebtables.ebtables_available_tables:
+           "filter" not in self.get_available_tables("eb"):
             log.error("ebtables not usable, disabling ethernet bridge firewall.")
             self.ebtables_enabled = False
 
+        # is there at least support for ipv4 or ipv6
         if not self.ip4tables_enabled and not self.ip6tables_enabled:
             log.fatal("No IPv4 and IPv6 firewall.")
             sys.exit(1)
 
     def _start_check(self):
         try:
-            x = self._ipset.list()
+            x = self.ipset_backend.list()
         except:
             log.error("ipset not usable, disabling ipset usage in firewall.")
+            # ipset is not usable, no supported types
             self.ipset_enabled = False
+            self.ipset_supported_types = [ ]
+        else:
+            # ipset is usable, get all supported types
+            self.ipset_supported_types = self.ipset_backend.supported_types()
 
-    def _start(self):
+    def _start(self, reload=False, complete_reload=False):
         # initialize firewall
-        default_zone = FALLBACK_ZONE
+        default_zone = config.FALLBACK_ZONE
 
         # load firewalld config
-        log.debug1("Loading firewalld config file '%s'", FIREWALLD_CONF)
+        log.debug1("Loading firewalld config file '%s'", config.FIREWALLD_CONF)
         try:
             self._firewalld_conf.read()
         except Exception as msg:
@@ -181,14 +199,13 @@ class Firewall(object):
                     self._log_denied = value.lower()
                     log.debug1("LogDenied is set to '%s'", self._log_denied)
 
-            if not self._individual_calls and \
-               not self._ebtables.restore_noflush_option:
+            if self.ebtables_enabled and not self._individual_calls and \
+               not self.ebtables_backend.restore_noflush_option:
                 log.debug1("ebtables-restore is not supporting the --noflush option, will therefore not be used")
 
         self.config.set_firewalld_conf(copy.deepcopy(self._firewalld_conf))
 
-        # apply default rules
-        self._apply_default_rules()
+        self._start_check()
 
         # load lockdown whitelist
         log.debug1("Loading lockdown whitelist")
@@ -206,26 +223,26 @@ class Firewall(object):
         self.config.set_policies(copy.deepcopy(self.policies))
 
         # load ipset files
-        self._loader(FIREWALLD_IPSETS, "ipset")
-        self._loader(ETC_FIREWALLD_IPSETS, "ipset")
+        self._loader(config.FIREWALLD_IPSETS, "ipset")
+        self._loader(config.ETC_FIREWALLD_IPSETS, "ipset")
 
         # load icmptype files
-        self._loader(FIREWALLD_ICMPTYPES, "icmptype")
-        self._loader(ETC_FIREWALLD_ICMPTYPES, "icmptype")
+        self._loader(config.FIREWALLD_ICMPTYPES, "icmptype")
+        self._loader(config.ETC_FIREWALLD_ICMPTYPES, "icmptype")
 
         if len(self.icmptype.get_icmptypes()) == 0:
             log.error("No icmptypes found.")
 
         # load service files
-        self._loader(FIREWALLD_SERVICES, "service")
-        self._loader(ETC_FIREWALLD_SERVICES, "service")
+        self._loader(config.FIREWALLD_SERVICES, "service")
+        self._loader(config.ETC_FIREWALLD_SERVICES, "service")
 
         if len(self.service.get_services()) == 0:
             log.error("No services found.")
 
         # load zone files
-        self._loader(FIREWALLD_ZONES, "zone")
-        self._loader(ETC_FIREWALLD_ZONES, "zone")
+        self._loader(config.FIREWALLD_ZONES, "zone")
+        self._loader(config.ETC_FIREWALLD_ZONES, "zone")
 
         if len(self.zone.get_zones()) == 0:
             log.fatal("No zones found.")
@@ -240,24 +257,6 @@ class Firewall(object):
         if error:
             sys.exit(1)
 
-        # apply settings for loaded ipsets
-        self.ipset.apply_ipsets(self._individual_calls)
-
-        # apply settings for loaded zones
-        self.zone.apply_zones()
-
-        # load direct rules
-        obj = Direct(FIREWALLD_DIRECT)
-        if os.path.exists(FIREWALLD_DIRECT):
-            log.debug1("Loading direct rules file '%s'" % FIREWALLD_DIRECT)
-            try:
-                obj.read()
-            except Exception as msg:
-                log.debug1("Failed to load direct rules file '%s': %s",
-                           FIREWALLD_DIRECT, msg)
-        self.direct.set_permanent_config(obj)
-        self.config.set_direct(copy.deepcopy(obj))
-
         # check if default_zone is a valid zone
         if default_zone not in self.zone.get_zones():
             if "public" in self.zone.get_zones():
@@ -273,17 +272,58 @@ class Firewall(object):
         else:
             log.debug1("Using default zone '%s'", default_zone)
 
+        # load direct rules
+        obj = Direct(config.FIREWALLD_DIRECT)
+        if os.path.exists(config.FIREWALLD_DIRECT):
+            log.debug1("Loading direct rules file '%s'" % \
+                       config.FIREWALLD_DIRECT)
+            try:
+                obj.read()
+            except Exception as msg:
+                log.debug1("Failed to load direct rules file '%s': %s",
+                           config.FIREWALLD_DIRECT, msg)
+        self.direct.set_permanent_config(obj)
+        self.config.set_direct(copy.deepcopy(obj))
+
+        if reload:
+            self._set_policy("DROP")
+
+        # check if needed tables are there
+        self._check_tables()
+
+        # flush rules
+        self._flush()
+
+        # complete reload: unload modules also
+        if reload and complete_reload:
+            log.debug1("Unloading firewall modules")
+            self.modules_backend.unload_firewall_modules()
+
+        # apply default rules
+        log.debug1("Applying default rule set")
+        self._apply_default_rules()
+
+        # apply settings for loaded ipsets
+        if self.ipset_enabled:
+            log.debug1("Applying ipsets")
+            self.ipset.apply_ipsets()
+
+        # apply settings for loaded zones
+        log.debug1("Applying used zones")
+        self.zone.apply_zones()
+
         self._default_zone = self.check_zone(default_zone)
         self.zone.change_default_zone(None, self._default_zone)
 
+        # apply direct chains, rules and passthrough rules
+        log.debug1("Applying direct chains rules and and passthrough rules")
+        self.direct.apply_direct()
+
         self._state = "RUNNING"
 
     def start(self):
-        self._start_check()
-        self._check_tables()
-        self._flush()
-        self._set_policy("ACCEPT")
         self._start()
+        self._set_policy("ACCEPT")
 
     def _loader(self, path, reader_type, combine=False):
         # combine: several zone files are getting combined into one obj
@@ -291,7 +331,7 @@ class Firewall(object):
             return
 
         if combine:
-            if path.startswith(ETC_FIREWALLD) and reader_type == "zone":
+            if path.startswith(config.ETC_FIREWALLD) and reader_type == "zone":
                 combined_zone = Zone()
                 combined_zone.name = os.path.basename(path)
                 combined_zone.check_name(combined_zone.name)
@@ -302,7 +342,7 @@ class Firewall(object):
 
         for filename in sorted(os.listdir(path)):
             if not filename.endswith(".xml"):
-                if path.startswith(ETC_FIREWALLD) and \
+                if path.startswith(config.ETC_FIREWALLD) and \
                         reader_type == "zone" and \
                         os.path.isdir("%s/%s" % (path, filename)):
                     self._loader("%s/%s" % (path, filename), reader_type,
@@ -320,7 +360,7 @@ class Firewall(object):
                                    orig_obj.name, orig_obj.path,
                                    orig_obj.filename)
                         self.icmptype.remove_icmptype(orig_obj.name)
-                    elif obj.path.startswith(ETC_FIREWALLD):
+                    elif obj.path.startswith(config.ETC_FIREWALLD):
                         obj.default = True
                     self.icmptype.add_icmptype(obj)
                     # add a deep copy to the configuration interface
@@ -333,7 +373,7 @@ class Firewall(object):
                                    orig_obj.name, orig_obj.path,
                                    orig_obj.filename)
                         self.service.remove_service(orig_obj.name)
-                    elif obj.path.startswith(ETC_FIREWALLD):
+                    elif obj.path.startswith(config.ETC_FIREWALLD):
                         obj.default = True
                     self.service.add_service(obj)
                     # add a deep copy to the configuration interface
@@ -361,7 +401,7 @@ class Firewall(object):
                                        reader_type,
                                        orig_obj.name, orig_obj.path,
                                        orig_obj.filename)
-                    elif obj.path.startswith(ETC_FIREWALLD):
+                    elif obj.path.startswith(config.ETC_FIREWALLD):
                         obj.default = True
                         config_obj.default = True
                     self.config.add_zone(config_obj)
@@ -380,7 +420,7 @@ class Firewall(object):
                                    orig_obj.name, orig_obj.path,
                                    orig_obj.filename)
                         self.ipset.remove_ipset(orig_obj.name)
-                    elif obj.path.startswith(ETC_FIREWALLD):
+                    elif obj.path.startswith(config.ETC_FIREWALLD):
                         obj.default = True
                     self.ipset.add_ipset(obj)
                     # add a deep copy to the configuration interface
@@ -407,6 +447,11 @@ class Firewall(object):
                 self.config.forget_zone(combined_zone.name)
             self.zone.add_zone(combined_zone)
 
+    def get_available_tables(self, ipv):
+        if ipv in [ "ipv4", "ipv6", "eb" ]:
+            return self.available_tables[ipv]
+        return [ ]
+
     def cleanup(self):
         self.icmptype.cleanup()
         self.service.cleanup()
@@ -422,7 +467,7 @@ class Firewall(object):
         if self.cleanup_on_exit:
             self._flush()
             self._set_policy("ACCEPT")
-            self._modules.unload_firewall_modules()
+            self.modules_backend.unload_firewall_modules()
 
         self.cleanup()
 
@@ -471,22 +516,24 @@ class Firewall(object):
             if insert and not enable and isinstance(rule[1], int):
                 rule.pop(1)
 
-            if table and not self.is_table_available(ipv, table):
+            if table and table not in self.get_available_tables(ipv):
                 if ((ipv == "ipv4" and self.ip4tables_enabled) or
                     (ipv == "ipv6" and self.ip6tables_enabled)):
-                    log.error("Unable to add %s into %s %s" % (rule, ipv, table))
+                    log.error("Unable to add %s into %s %s" % (rule, ipv,
+                                                               table))
                 continue
 
-            if table != None:
+            if table is not None:
                 _rule = [ "-t", table, append_delete[enable], ]
             else:
                 _rule = [ append_delete[enable], ]
-            if chain != None:
+            if chain is not None:
                 _rule.append(chain)
             _rule += [ "%s" % item for item in rule ]
 
             if self._individual_calls or \
-               (ipv == "eb" and not self._ebtables.restore_noflush_option):
+               (ipv == "eb" and not
+                self.ebtables_backend.restore_noflush_option):
                 ## run
                 try:
                     self.rule(ipv, _rule)
@@ -516,7 +563,8 @@ class Firewall(object):
         for i,(ipv, rule) in enumerate(rules):
             _rule = [ new_delete[enable], ] + rule
             if self._individual_calls or \
-               (ipv == "eb" and not self._ebtables.restore_noflush_option):
+               (ipv == "eb" and not
+                self.ebtables_backend.restore_noflush_option):
                 try:
                     self.rule(ipv, _rule)
                 except Exception as msg:
@@ -536,12 +584,12 @@ class Firewall(object):
     def handle_modules(self, modules, enable):
         for i,module in enumerate(modules):
             if enable:
-                (status, msg) = self._modules.load_module(module)
+                (status, msg) = self.modules_backend.load_module(module)
             else:
                 if self._module_refcount[module] > 1:
                     status = 0 # module referenced more then one, do not unload
                 else:
-                    (status, msg) = self._modules.unload_module(module)
+                    (status, msg) = self.modules_backend.unload_module(module)
             if status != 0:
                 if enable:
                     return (modules[:i], msg) # cleanup modules and error msg
@@ -557,11 +605,6 @@ class Firewall(object):
                         del self._module_refcount[module]
         return None
 
-    def is_table_available(self, ipv, table):
-        return ((ipv == "ipv4" and table in ipXtables.ip4tables_available_tables) or
-                (ipv == "ipv6" and table in ipXtables.ip6tables_available_tables) or
-                (ipv == "eb" and table in ebtables.ebtables_available_tables))
-
     # apply default rules
     def __apply_default_rules(self, ipv):
         default_rules = { }
@@ -579,7 +622,7 @@ class Firewall(object):
 
         rules = { }
         for table in default_rules:
-            if not self.is_table_available(ipv, table):
+            if table not in self.get_available_tables(ipv):
                 continue
             prefix = [ "-t", table ]
             for rule in default_rules[table]:
@@ -588,7 +631,8 @@ class Firewall(object):
                 else:
                     _rule = prefix + functions.splitArgs(rule)
                 if self._individual_calls or \
-                   (ipv == "eb" and not self._ebtables.restore_noflush_option):
+                   (ipv == "eb" and not
+                    self.ebtables_backend.restore_noflush_option):
                     self.rule(ipv, _rule)
                 else:
                     rules.setdefault(ipv, []).append(_rule)
@@ -601,12 +645,13 @@ class Firewall(object):
             self.__apply_default_rules(ipv)
 
         if self.ipv6_rpfilter_enabled and \
-           self.is_table_available("ipv6", "raw"):
+           "raw" in self.get_available_tables("ipv6"):
             # here is no check for ebtables.restore_noflush_option needed
             # as ebtables is not used in here
             rules = [
                 ("ipv6", [ "PREROUTING", 1, "-t", "raw",
-                           "-p", "icmpv6", "--icmpv6-type=router-advertisement",
+                           "-p", "ipv6-icmp",
+                           "--icmpv6-type=router-advertisement",
                            "-j", "ACCEPT" ]), # RHBZ#1058505
                 ("ipv6", [ "PREROUTING", 2, "-t", "raw",
                            "-m", "rpfilter", "--invert", "-j", "DROP" ]),
@@ -626,25 +671,50 @@ class Firewall(object):
     # flush and policy
 
     def _flush(self):
+        log.debug1("Flushing rule set")
+
         if self.ip4tables_enabled:
-            self._ip4tables.flush(individual=self._individual_calls)
+            try:
+                self.ip4tables_backend.flush(individual=self._individual_calls)
+            except Exception as e:
+                log.error("Failed to flush ipv4 firewall: %s" % e)
         if self.ip6tables_enabled:
-            self._ip6tables.flush(individual=self._individual_calls)
+            try:
+                self.ip6tables_backend.flush(individual=self._individual_calls)
+            except Exception as e:
+                log.error("Failed to flush ipv6 firewall: %s" % e)
         if self.ebtables_enabled:
-            self._ebtables.flush(individual=(self._individual_calls or \
-                                             not self._ebtables.restore_noflush_option))
+            try:
+                self.ebtables_backend.flush(
+                    individual=self._individual_calls or \
+                    not self.ebtables_backend.restore_noflush_option)
+            except Exception as e:
+                log.error("Failed to flush eb firewall: %s" % e)
 
     def _set_policy(self, policy, which="used"):
+        log.debug1("Setting policy to '%s'", policy)
+
         if self.ip4tables_enabled:
-            self._ip4tables.set_policy(policy, which,
-                                       individual=self._individual_calls)
+            try:
+                self.ip4tables_backend.set_policy(
+                    policy, which, individual=self._individual_calls)
+            except Exception as e:
+                log.error("Failed to set policy of ipv4 firewall: %s" % e)
+
         if self.ip6tables_enabled:
-            self._ip6tables.set_policy(policy, which,
-                                       individual=self._individual_calls)
+            try:
+                self.ip6tables_backend.set_policy(
+                    policy, which, individual=self._individual_calls)
+            except Exception as e:
+                log.error("Failed to set policy of ipv6 firewall: %s" % e)
         if self.ebtables_enabled:
-            self._ebtables.set_policy(policy, which,
-                                      individual=(self._individual_calls or \
-                                                  not self._ebtables.restore_noflush_option))
+            try:
+                self.ebtables_backend.set_policy(
+                    policy, which,
+                    individual=self._individual_calls or \
+                    not self.ebtables_backend.restore_noflush_option)
+            except Exception as e:
+                log.error("Failed to set policy of eb firewall: %s" % e)
 
     # rule function used in handle_ functions
 
@@ -659,7 +729,7 @@ class Firewall(object):
                 rule[i:i+1] = [ "REJECT", "--reject-with",
                                 ipXtables.DEFAULT_REJECT_TYPE[ipv] ]
             else:
-                raise FirewallError(EBTABLES_NO_REJECT,
+                raise FirewallError(errors.EBTABLES_NO_REJECT,
                                     "'%s' not in {'ipv4'|'ipv6'}" % ipv)
 
         # replace %%ICMP%%
@@ -671,7 +741,7 @@ class Firewall(object):
             if ipv in [ "ipv4", "ipv6" ]:
                 rule[i] = ipXtables.ICMP[ipv]
             else:
-                raise FirewallError(INVALID_IPV,
+                raise FirewallError(errors.INVALID_IPV,
                                     "'%s' not in {'ipv4'|'ipv6'}" % ipv)
 
         # replace %%LOGTYPE%%
@@ -683,7 +753,7 @@ class Firewall(object):
             if self._log_denied == "off":
                 return ""
             if ipv not in [ "ipv4", "ipv6" ]:
-                raise FirewallError(INVALID_IPV,
+                raise FirewallError(errors.INVALID_IPV,
                                     "'%s' not in {'ipv4'|'ipv6'}" % ipv)
             if self._log_denied in [ "unicast", "broadcast", "multicast" ]:
                 rule[i:i+1] = [ "-m", "pkttype", "--pkt-type",
@@ -702,17 +772,17 @@ class Firewall(object):
         if ipv == "ipv4":
             # do not call if disabled
             if self.ip4tables_enabled:
-                return self._ip4tables.set_rule(rule)
+                return self.ip4tables_backend.set_rule(rule)
         elif ipv == "ipv6":
             # do not call if disabled
             if self.ip6tables_enabled:
-                return self._ip6tables.set_rule(rule)
+                return self.ip6tables_backend.set_rule(rule)
         elif ipv == "eb":
             # do not call if disabled
             if self.ebtables_enabled:
-                return self._ebtables.set_rule(rule)
+                return self.ebtables_backend.set_rule(rule)
         else:
-            raise FirewallError(INVALID_IPV,
+            raise FirewallError(errors.INVALID_IPV,
                                 "'%s' not in {'ipv4'|'ipv6'|'eb'}" % ipv)
 
         return ""
@@ -731,7 +801,7 @@ class Firewall(object):
                     rule[i:i+1] = [ "REJECT", "--reject-with",
                                     ipXtables.DEFAULT_REJECT_TYPE[ipv] ]
                 else:
-                    raise FirewallError(EBTABLES_NO_REJECT,
+                    raise FirewallError(errors.EBTABLES_NO_REJECT,
                                         "'%s' not in {'ipv4'|'ipv6'}" % ipv)
 
             # replace %%ICMP%%
@@ -743,7 +813,7 @@ class Firewall(object):
                 if ipv in [ "ipv4", "ipv6" ]:
                     rule[i] = ipXtables.ICMP[ipv]
                 else:
-                    raise FirewallError(INVALID_IPV,
+                    raise FirewallError(errors.INVALID_IPV,
                                         "'%s' not in {'ipv4'|'ipv6'}" % ipv)
 
             # replace %%LOGTYPE%%
@@ -755,7 +825,7 @@ class Firewall(object):
                 if self._log_denied == "off":
                     continue
                 if ipv not in [ "ipv4", "ipv6" ]:
-                    raise FirewallError(INVALID_IPV,
+                    raise FirewallError(errors.INVALID_IPV,
                                     "'%s' not in {'ipv4'|'ipv6'}" % ipv)
                 if self._log_denied in [ "unicast", "broadcast",
                                          "multicast" ]:
@@ -769,17 +839,17 @@ class Firewall(object):
         if ipv == "ipv4":
             # do not call if disabled
             if self.ip4tables_enabled:
-                return self._ip4tables.set_rules(_rules)
+                return self.ip4tables_backend.set_rules(_rules)
         elif ipv == "ipv6":
             # do not call if disabled
             if self.ip6tables_enabled:
-                return self._ip6tables.set_rules(_rules)
+                return self.ip6tables_backend.set_rules(_rules)
         elif ipv == "eb":
             # do not call if disabled
             if self.ebtables_enabled:
-                return self._ebtables.set_rules(_rules)
+                return self.ebtables_backend.set_rules(_rules)
         else:
-            raise FirewallError(INVALID_IPV,
+            raise FirewallError(errors.INVALID_IPV,
                                 "'%s' not in {'ipv4'|'ipv6'|'eb'}" % ipv)
 
         return ""
@@ -788,19 +858,19 @@ class Firewall(object):
 
     def check_panic(self):
         if self._panic:
-            raise FirewallError(PANIC_MODE)
+            raise FirewallError(errors.PANIC_MODE)
 
     def check_zone(self, zone):
         _zone = zone
         if not _zone or _zone == "":
             _zone = self.get_default_zone()
         if _zone not in self.zone.get_zones():
-            raise FirewallError(INVALID_ZONE, _zone)
+            raise FirewallError(errors.INVALID_ZONE, _zone)
         return _zone
 
     def check_interface(self, interface):
         if not functions.checkInterface(interface):
-            raise FirewallError(INVALID_INTERFACE, interface)
+            raise FirewallError(errors.INVALID_INTERFACE, interface)
 
     def check_service(self, service):
         self.service.check_service(service)
@@ -818,28 +888,28 @@ class Firewall(object):
                 log.debug1("'%s': port is ambiguous" % port)
             elif len(range) == 2 and range[0] >= range[1]:
                 log.debug1("'%s': range start >= end" % port)
-            raise FirewallError(INVALID_PORT, port)
+            raise FirewallError(errors.INVALID_PORT, port)
 
     def check_tcpudp(self, protocol):
         if not protocol:
-            raise FirewallError(MISSING_PROTOCOL)
+            raise FirewallError(errors.MISSING_PROTOCOL)
         if not protocol in [ "tcp", "udp" ]:
-            raise FirewallError(INVALID_PROTOCOL,
+            raise FirewallError(errors.INVALID_PROTOCOL,
                                 "'%s' not in {'tcp'|'udp'}" % protocol)
 
     def check_ip(self, ip):
         if not functions.checkIP(ip):
-            raise FirewallError(INVALID_ADDR, ip)
+            raise FirewallError(errors.INVALID_ADDR, ip)
 
     def check_address(self, ipv, source):
         if ipv == "ipv4":
             if not functions.checkIPnMask(source):
-                raise FirewallError(INVALID_ADDR, source)
+                raise FirewallError(errors.INVALID_ADDR, source)
         elif ipv == "ipv6":
             if not functions.checkIP6nMask(source):
-                raise FirewallError(INVALID_ADDR, source)
+                raise FirewallError(errors.INVALID_ADDR, source)
         else:
-            raise FirewallError(INVALID_IPV,
+            raise FirewallError(errors.INVALID_IPV,
                                 "'%s' not in {'ipv4'|'ipv6'}")
 
     def check_icmptype(self, icmp):
@@ -849,7 +919,7 @@ class Firewall(object):
         if not isinstance(timeout, int):
             raise TypeError("%s is %s, expected int" % (timeout, type(timeout)))
         if int(timeout) < 0:
-            raise FirewallError(INVALID_VALUE,
+            raise FirewallError(errors.INVALID_VALUE,
                                 "timeout '%d' is not positive number" % timeout)
 
     # RELOAD
@@ -866,14 +936,10 @@ class Firewall(object):
         _old_dz = self.get_default_zone()
 
         # stop
-        self._set_policy("DROP")
-        self._flush()
-        if stop:
-            self._modules.unload_firewall_modules()
         self.cleanup()
 
         # start
-        self._start()
+        self._start(reload=True, complete_reload=stop)
 
         # handle interfaces in the default zone and move them to the new 
         # default zone if it changed
@@ -927,26 +993,26 @@ class Firewall(object):
 
     def enable_panic_mode(self):
         if self._panic:
-            raise FirewallError(ALREADY_ENABLED,
+            raise FirewallError(errors.ALREADY_ENABLED,
                                 "panic mode already enabled")
 
         # TODO: use rule in raw table not default chain policy
         try:
             self._set_policy("DROP", "all")
         except Exception as msg:
-            raise FirewallError(COMMAND_FAILED, msg)
+            raise FirewallError(errors.COMMAND_FAILED, msg)
         self._panic = True
 
     def disable_panic_mode(self):
         if not self._panic:
-            raise FirewallError(NOT_ENABLED,
+            raise FirewallError(errors.NOT_ENABLED,
                                 "panic mode is not enabled")
 
         # TODO: use rule in raw table not default chain policy
         try:
             self._set_policy("ACCEPT", "all")
         except Exception as msg:
-            raise FirewallError(COMMAND_FAILED, msg)
+            raise FirewallError(errors.COMMAND_FAILED, msg)
         self._panic = False
 
     def query_panic_mode(self):
@@ -958,9 +1024,10 @@ class Firewall(object):
         return self._log_denied
 
     def set_log_denied(self, value):
-        if value not in LOG_DENIED_VALUES:
-            raise FirewallError(INVALID_VALUE, "'%s', choose from '%s'" % \
-                                (value, "','".join(LOG_DENIED_VALUES)))
+        if value not in config.LOG_DENIED_VALUES:
+            raise FirewallError(errors.INVALID_VALUE,
+                                "'%s', choose from '%s'" % \
+                                (value, "','".join(config.LOG_DENIED_VALUES)))
 
         if value != self.get_log_denied():
             self._log_denied = value
@@ -970,7 +1037,7 @@ class Firewall(object):
             # now reload the firewall
             self.reload()
         else:
-            raise FirewallError(ALREADY_SET, value)
+            raise FirewallError(errors.ALREADY_SET, value)
 
     # DEFAULT ZONE
 
@@ -996,4 +1063,4 @@ class Firewall(object):
                     # (not those that were added to specific zone same as default)
                     self.zone.change_zone_of_interface("", iface)
         else:
-            raise FirewallError(ZONE_ALREADY_SET, _zone)
+            raise FirewallError(errors.ZONE_ALREADY_SET, _zone)
