@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2011-2012 Red Hat, Inc.
+# Copyright (C) 2011-2016 Red Hat, Inc.
 #
 # Authors:
 # Thomas Woerner <twoerner@redhat.com>
@@ -19,16 +19,20 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
+__all__ = [ "FirewallConfig" ]
+
 import copy
 import os, os.path
-from firewall.config import *
+from firewall import config
 from firewall.core.base import *
 from firewall.core.logger import log
 from firewall.core.io.icmptype import IcmpType, icmptype_reader, icmptype_writer
 from firewall.core.io.service import Service, service_reader, service_writer
 from firewall.core.io.zone import Zone, zone_reader, zone_writer
+from firewall.core.io.ipset import IPSet, ipset_reader, ipset_writer
 from firewall.functions import portStr
-from firewall.errors import *
+from firewall import errors
+from firewall.errors import FirewallError
 
 class FirewallConfig(object):
     def __init__(self, fw):
@@ -36,40 +40,49 @@ class FirewallConfig(object):
         self.__init_vars()
 
     def __repr__(self):
-        return '%s(%r, %r, %r, %r, %r, %r, %r, %r, %r)' % (self.__class__,
-                   self._icmptypes, self._services, self._zones,
-                   self._default_icmptypes, self._default_services, self._default_zones,
+        return '%s(%r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r)' % (self.__class__,
+                   self._ipsets, self._icmptypes, self._services, self._zones,
+                   self._builtin_ipsets, self._builtin_icmptypes, self._builtin_services, self._builtin_zones,
                    self._firewalld_conf, self._policies, self._direct)
 
     def __init_vars(self):
+        self._ipsets = { }
         self._icmptypes = { }
         self._services = { }
         self._zones = { }
-        self._default_icmptypes = { }
-        self._default_services = { }
-        self._default_zones = { }
+        self._builtin_ipsets = { }
+        self._builtin_icmptypes = { }
+        self._builtin_services = { }
+        self._builtin_zones = { }
         self._firewalld_conf = None
         self._policies = None
         self._direct = None
 
     def cleanup(self):
-        for x in list(self._default_icmptypes.keys()):
-            self._default_icmptypes[x].cleanup()
-            del self._default_icmptypes[x]
+        for x in list(self._builtin_ipsets.keys()):
+            self._builtin_ipsets[x].cleanup()
+            del self._builtin_ipsets[x]
+        for x in list(self._ipsets.keys()):
+            self._ipsets[x].cleanup()
+            del self._ipsets[x]
+
+        for x in list(self._builtin_icmptypes.keys()):
+            self._builtin_icmptypes[x].cleanup()
+            del self._builtin_icmptypes[x]
         for x in list(self._icmptypes.keys()):
             self._icmptypes[x].cleanup()
             del self._icmptypes[x]
 
-        for x in list(self._default_services.keys()):
-            self._default_services[x].cleanup()
-            del self._default_services[x]
+        for x in list(self._builtin_services.keys()):
+            self._builtin_services[x].cleanup()
+            del self._builtin_services[x]
         for x in list(self._services.keys()):
             self._services[x].cleanup()
             del self._services[x]
 
-        for x in list(self._default_zones.keys()):
-            self._default_zones[x].cleanup()
-            del self._default_zones[x]
+        for x in list(self._builtin_zones.keys()):
+            self._builtin_zones[x].cleanup()
+            del self._builtin_zones[x]
         for x in list(self._zones.keys()):
             self._zones[x].cleanup()
             del self._zones[x]
@@ -108,7 +121,7 @@ class FirewallConfig(object):
         return self._firewalld_conf
 
     def update_firewalld_conf(self):
-        if not os.path.exists(FIREWALLD_CONF):
+        if not os.path.exists(config.FIREWALLD_CONF):
             self._firewalld_conf.clear()
         else:
             self._firewalld_conf.read()
@@ -122,7 +135,7 @@ class FirewallConfig(object):
         return self._policies
 
     def update_lockdown_whitelist(self):
-        if not os.path.exists(LOCKDOWN_WHITELIST):
+        if not os.path.exists(config.LOCKDOWN_WHITELIST):
             self._policies.lockdown_whitelist.cleanup()
         else:
             self._policies.lockdown_whitelist.read()
@@ -136,81 +149,248 @@ class FirewallConfig(object):
         return self._direct
 
     def update_direct(self):
-        if not os.path.exists(FIREWALLD_DIRECT):
+        if not os.path.exists(config.FIREWALLD_DIRECT):
             self._direct.cleanup()
         else:
             self._direct.read()
 
+    # ipset
+
+    def get_ipsets(self):
+        return sorted(set(list(self._ipsets.keys()) + \
+                          list(self._builtin_ipsets.keys())))
+
+    def add_ipset(self, obj):
+        if obj.builtin:
+            self._builtin_ipsets[obj.name] = obj
+        else:
+            self._ipsets[obj.name] = obj
+
+    def get_ipset(self, name):
+        if name in self._ipsets:
+            return self._ipsets[name]
+        elif name in self._builtin_ipsets:
+            return self._builtin_ipsets[name]
+        raise FirewallError(errors.INVALID_IPSET, name)
+
+    def load_ipset_defaults(self, obj):
+        if obj.name not in self._ipsets:
+            raise FirewallError(errors.NO_DEFAULTS, obj.name)
+        elif self._ipsets[obj.name] != obj:
+            raise FirewallError(errors.NO_DEFAULTS,
+                                "self._ipsets[%s] != obj" % obj.name)
+        elif obj.name not in self._builtin_ipsets:
+            raise FirewallError(errors.NO_DEFAULTS,
+                            "'%s' not in self._builtin_ipsets" % obj.name)
+        self._remove_ipset(obj)
+        return self._builtin_ipsets[obj.name]
+
+    def get_ipset_config(self, obj):
+        return obj.export_config()
+
+    def set_ipset_config(self, obj, conf):
+        if obj.builtin:
+            x = copy.copy(obj)
+            x.import_config(conf)
+            x.path = config.ETC_FIREWALLD_IPSETS
+            x.builtin = False
+            if obj.path != x.path:
+                x.default = False
+            self.add_ipset(x)
+            ipset_writer(x)
+            return x
+        else:
+            obj.import_config(conf)
+            ipset_writer(obj)
+            return obj
+
+    def new_ipset(self, name, conf):
+        try:
+            self.get_ipset(name)
+        except:
+            pass
+        else:
+            raise FirewallError(errors.NAME_CONFLICT, "new_ipset(): '%s'" % name)
+
+        x = IPSet()
+        x.check_name(name)
+        x.import_config(conf)
+        x.name = name
+        x.filename = "%s.xml" % name
+        x.path = config.ETC_FIREWALLD_IPSETS
+        # It is not possible to add a new one with a name of a buitin
+        x.builtin = False
+        x.default = True
+
+        ipset_writer(x)
+        self.add_ipset(x)
+        return x
+
+    def update_ipset_from_path(self, name):
+        filename = os.path.basename(name)
+        path = os.path.dirname(name)
+
+        if not os.path.exists(name):
+            # removed file
+
+            if path == config.ETC_FIREWALLD_IPSETS:
+                # removed custom ipset
+                for x in self._ipsets.keys():
+                    obj = self._ipsets[x]
+                    if obj.filename == filename:
+                        del self._ipsets[x]
+                        if obj.name in self._builtin_ipsets:
+                            return ("update", self._builtin_ipsets[obj.name])
+                        return ("remove", obj)
+            else:
+                # removed builtin ipset
+                for x in self._builtin_ipsets.keys():
+                    obj = self._builtin_ipsets[x]
+                    if obj.filename == filename:
+                        del self._builtin_ipsets[x]
+                        if obj.name not in self._ipsets:
+                            # update dbus ipset
+                            return ("remove", obj)
+                        else:
+                            # builtin hidden, no update needed
+                            return (None, None)
+
+            # ipset not known to firewalld, yet (timeout, ..)
+            return (None, None)
+
+        # new or updated file
+
+        log.debug1("Loading ipset file '%s'", name)
+        try:
+            obj = ipset_reader(filename, path)
+        except Exception as msg:
+            log.error("Failed to load ipset file '%s': %s", filename, msg)
+            return (None, None)
+
+        # new ipset
+        if obj.name not in self._builtin_ipsets and obj.name not in self._ipsets:
+            self.add_ipset(obj)
+            return ("new", obj)
+
+        # updated ipset
+        if path == config.ETC_FIREWALLD_IPSETS:
+            # custom ipset update
+            if obj.name in self._ipsets:
+                obj.default = self._ipsets[obj.name].default
+                self._ipsets[obj.name] = obj
+            return ("update", obj)
+        else:
+            if obj.name in self._builtin_ipsets:
+                # builtin ipset update
+                del self._builtin_ipsets[obj.name]
+                self._builtin_ipsets[obj.name] = obj
+
+                if obj.name not in self._ipsets:
+                    # update dbus ipset
+                    return ("update", obj)
+                else:
+                    # builtin hidden, no update needed
+                    return (None, None)
+
+        # ipset not known to firewalld, yet (timeout, ..)
+        return (None, None)
+
+    def _remove_ipset(self, obj):
+        if obj.name not in self._ipsets:
+            raise FirewallError(errors.INVALID_IPSET,
+                                "'%s' not in self._ipsets" % obj.name)
+        if obj.path != config.ETC_FIREWALLD_IPSETS:
+            raise FirewallError(errors.INVALID_DIRECTORY,
+                                "'%s' != '%s'" % (obj.path,
+                                                  config.ETC_FIREWALLD_IPSETS))
+        os.remove("%s/%s.xml" % (obj.path, obj.name))
+        del self._ipsets[obj.name]
+
+    def check_builtin_ipset(self, obj):
+        if obj.builtin or not obj.default:
+            raise FirewallError(errors.BUILTIN_IPSET,
+                                "'%s' is built-in ipset" % obj.name)
+
+    def remove_ipset(self, obj):
+        self.check_builtin_ipset(obj)
+        self._remove_ipset(obj)
+
+    def rename_ipset(self, obj, name):
+        self.check_builtin_ipset(obj)
+        new_ipset = self._copy_ipset(obj, name)
+        self._remove_ipset(obj)
+        return new_ipset
+
+    def _copy_ipset(self, obj, name):
+        return self.new_ipset(name, obj.export_config())
+
     # icmptypes
 
     def get_icmptypes(self):
         return sorted(set(list(self._icmptypes.keys()) + \
-                          list(self._default_icmptypes.keys())))
+                          list(self._builtin_icmptypes.keys())))
 
     def add_icmptype(self, obj):
-        if obj.default:
-            self._default_icmptypes[obj.name] = obj
+        if obj.builtin:
+            self._builtin_icmptypes[obj.name] = obj
         else:
             self._icmptypes[obj.name] = obj
 
     def get_icmptype(self, name):
         if name in self._icmptypes:
             return self._icmptypes[name]
-        elif name in self._default_icmptypes:
-            return self._default_icmptypes[name]
-        raise FirewallError(INVALID_ICMPTYPE, name)
-
-    def icmptype_has_defaults(self, name):
-        return (name in self._icmptypes and name in self._default_icmptypes)
-
-    def icmptype_is_default(self, name):
-        return name in self._default_icmptypes
+        elif name in self._builtin_icmptypes:
+            return self._builtin_icmptypes[name]
+        raise FirewallError(errors.INVALID_ICMPTYPE, name)
 
     def load_icmptype_defaults(self, obj):
         if obj.name not in self._icmptypes:
-            raise FirewallError(NO_DEFAULTS,
-                                "'%s' not in self._icmptypes" % obj.name)
+            raise FirewallError(errors.NO_DEFAULTS, obj.name)
         elif self._icmptypes[obj.name] != obj:
-            raise FirewallError(NO_DEFAULTS,
+            raise FirewallError(errors.NO_DEFAULTS,
                                 "self._icmptypes[%s] != obj" % obj.name)
-        elif obj.name not in self._default_icmptypes:
-            raise FirewallError(NO_DEFAULTS,
-                            "'%s' not in self._default_icmptypes" % obj.name)
+        elif obj.name not in self._builtin_icmptypes:
+            raise FirewallError(errors.NO_DEFAULTS,
+                            "'%s' not in self._builtin_icmptypes" % obj.name)
         self._remove_icmptype(obj)
-        return self._default_icmptypes[obj.name]
+        return self._builtin_icmptypes[obj.name]
 
     def get_icmptype_config(self, obj):
         return obj.export_config()
 
-    def set_icmptype_config(self, obj, config):
-        if obj.default:
+    def set_icmptype_config(self, obj, conf):
+        if obj.builtin:
             x = copy.copy(obj)
-            x.import_config(config)
-            x.path = ETC_FIREWALLD_ICMPTYPES
-            x.default = False
+            x.import_config(conf)
+            x.path = config.ETC_FIREWALLD_ICMPTYPES
+            x.builtin = False
+            if obj.path != x.path:
+                x.default = False
             self.add_icmptype(x)
             icmptype_writer(x)
             return x
         else:
-            obj.import_config(config)
+            obj.import_config(conf)
             icmptype_writer(obj)
             return obj
 
-    def new_icmptype(self, name, config):
+    def new_icmptype(self, name, conf):
         try:
             self.get_icmptype(name)
         except:
             pass
         else:
-            raise FirewallError(NAME_CONFLICT, "new_icmptype(): '%s'" % name)
+            raise FirewallError(errors.NAME_CONFLICT, "new_icmptype(): '%s'" % name)
 
         x = IcmpType()
         x.check_name(name)
-        x.import_config(config)
+        x.import_config(conf)
         x.name = name
         x.filename = "%s.xml" % name
-        x.path = ETC_FIREWALLD_ICMPTYPES
-        x.default = False
+        x.path = config.ETC_FIREWALLD_ICMPTYPES
+        # It is not possible to add a new one with a name of a buitin
+        x.builtin = False
+        x.default = True
 
         icmptype_writer(x)
         self.add_icmptype(x)
@@ -223,21 +403,21 @@ class FirewallConfig(object):
         if not os.path.exists(name):
             # removed file
 
-            if path == ETC_FIREWALLD_ICMPTYPES:
+            if path == config.ETC_FIREWALLD_ICMPTYPES:
                 # removed custom icmptype
                 for x in self._icmptypes.keys():
                     obj = self._icmptypes[x]
                     if obj.filename == filename:
                         del self._icmptypes[x]
-                        if obj.name in self._default_icmptypes:
-                            return ("update", self._default_icmptypes[obj.name])
+                        if obj.name in self._builtin_icmptypes:
+                            return ("update", self._builtin_icmptypes[obj.name])
                         return ("remove", obj)
             else:
                 # removed builtin icmptype
-                for x in self._default_icmptypes.keys():
-                    obj = self._default_icmptypes[x]
+                for x in self._builtin_icmptypes.keys():
+                    obj = self._builtin_icmptypes[x]
                     if obj.filename == filename:
-                        del self._default_icmptypes[x]
+                        del self._builtin_icmptypes[x]
                         if obj.name not in self._icmptypes:
                             # update dbus icmptype
                             return ("remove", obj)
@@ -250,24 +430,30 @@ class FirewallConfig(object):
 
         # new or updated file
 
-        obj = icmptype_reader(filename, path)
+        log.debug1("Loading icmptype file '%s'", name)
+        try:
+            obj = icmptype_reader(filename, path)
+        except Exception as msg:
+            log.error("Failed to load icmptype file '%s': %s", filename, msg)
+            return (None, None)
 
         # new icmptype
-        if obj.name not in self._default_icmptypes and obj.name not in self._icmptypes:
+        if obj.name not in self._builtin_icmptypes and obj.name not in self._icmptypes:
             self.add_icmptype(obj)
             return ("new", obj)
 
         # updated icmptype
-        if path == ETC_FIREWALLD_ICMPTYPES:
+        if path == config.ETC_FIREWALLD_ICMPTYPES:
             # custom icmptype update
             if obj.name in self._icmptypes:
+                obj.default = self._icmptypes[obj.name].default
                 self._icmptypes[obj.name] = obj
             return ("update", obj)
         else:
-            if obj.name in self._default_icmptypes:
+            if obj.name in self._builtin_icmptypes:
                 # builtin icmptype update
-                del self._default_icmptypes[obj.name]
-                self._default_icmptypes[obj.name] = obj
+                del self._builtin_icmptypes[obj.name]
+                self._builtin_icmptypes[obj.name] = obj
 
                 if obj.name not in self._icmptypes:
                     # update dbus icmptype
@@ -281,22 +467,18 @@ class FirewallConfig(object):
 
     def _remove_icmptype(self, obj):
         if obj.name not in self._icmptypes:
-            raise FirewallError(INVALID_ICMPTYPE,
+            raise FirewallError(errors.INVALID_ICMPTYPE,
                                 "'%s' not in self._icmptypes" % obj.name)
-        if obj.path != ETC_FIREWALLD_ICMPTYPES:
-            raise FirewallError(INVALID_DIRECTORY,
-                        "'%s' != '%s'" % (obj.path, ETC_FIREWALLD_ICMPTYPES))
+        if obj.path != config.ETC_FIREWALLD_ICMPTYPES:
+            raise FirewallError(errors.INVALID_DIRECTORY,
+                                "'%s' != '%s'" % \
+                                (obj.path, config.ETC_FIREWALLD_ICMPTYPES))
         os.remove("%s/%s.xml" % (obj.path, obj.name))
         del self._icmptypes[obj.name]
 
-    def is_builtin_icmptype(self, obj):
-        if obj.default or obj.name in self._default_icmptypes:
-            return True
-        return False
-
     def check_builtin_icmptype(self, obj):
-        if self.is_builtin_icmptype(obj):
-            raise FirewallError(BUILTIN_ICMPTYPE,
+        if obj.builtin or not obj.default:
+            raise FirewallError(errors.BUILTIN_ICMPTYPE,
                                 "'%s' is built-in icmp type" % obj.name)
 
     def remove_icmptype(self, obj):
@@ -316,72 +498,69 @@ class FirewallConfig(object):
 
     def get_services(self):
         return sorted(set(list(self._services.keys()) + \
-                          list(self._default_services.keys())))
+                          list(self._builtin_services.keys())))
 
     def add_service(self, obj):
-        if obj.default:
-            self._default_services[obj.name] = obj
+        if obj.builtin:
+            self._builtin_services[obj.name] = obj
         else:
             self._services[obj.name] = obj
 
     def get_service(self, name):
         if name in self._services:
             return self._services[name]
-        elif name in self._default_services:
-            return self._default_services[name]
-        raise FirewallError(INVALID_SERVICE, "get_service(): '%s'" % name)
-
-    def service_has_defaults(self, name):
-        return (name in self._services and name in self._default_services)
-
-    def service_is_default(self, name):
-        return name in self._default_services
+        elif name in self._builtin_services:
+            return self._builtin_services[name]
+        raise FirewallError(errors.INVALID_SERVICE, "get_service(): '%s'" % name)
 
     def load_service_defaults(self, obj):
         if obj.name not in self._services:
-            raise FirewallError(NO_DEFAULTS,
-                                "'%s' not in self._services" % obj.name)
+            raise FirewallError(errors.NO_DEFAULTS, obj.name)
         elif self._services[obj.name] != obj:
-            raise FirewallError(NO_DEFAULTS,
+            raise FirewallError(errors.NO_DEFAULTS,
                                 "self._services[%s] != obj" % obj.name)
-        elif obj.name not in self._default_services:
-            raise FirewallError(NO_DEFAULTS,
-                            "'%s' not in self._default_services" % obj.name)
+        elif obj.name not in self._builtin_services:
+            raise FirewallError(errors.NO_DEFAULTS,
+                            "'%s' not in self._builtin_services" % obj.name)
         self._remove_service(obj)
-        return self._default_services[obj.name]
+        return self._builtin_services[obj.name]
 
     def get_service_config(self, obj):
         return obj.export_config()
 
-    def set_service_config(self, obj, config):
-        if obj.default:
+    def set_service_config(self, obj, conf):
+        if obj.builtin:
             x = copy.copy(obj)
-            x.import_config(config)
-            x.path = ETC_FIREWALLD_SERVICES
-            x.default = False
+            x.import_config(conf)
+            x.path = config.ETC_FIREWALLD_SERVICES
+            x.builtin = False
+            if obj.path != x.path:
+                x.default = False
             self.add_service(x)
             service_writer(x)
             return x
         else:
-            obj.import_config(config)
+            obj.import_config(conf)
             service_writer(obj)
             return obj
 
-    def new_service(self, name, config):
+    def new_service(self, name, conf):
         try:
             self.get_service(name)
         except:
             pass
         else:
-            raise FirewallError(NAME_CONFLICT, "new_service(): '%s'" % name)
+            raise FirewallError(errors.NAME_CONFLICT, "new_service(): '%s'" % name)
 
         x = Service()
         x.check_name(name)
-        x.import_config(config)
+        x.import_config(conf)
         x.name = name
         x.filename = "%s.xml" % name
-        x.path = ETC_FIREWALLD_SERVICES
-        x.default = False
+        x.path = config.ETC_FIREWALLD_SERVICES
+        # It is not possible to add a new one with a name of a buitin
+        x.builtin = False
+        x.default = True
 
         service_writer(x)
         self.add_service(x)
@@ -394,21 +573,21 @@ class FirewallConfig(object):
         if not os.path.exists(name):
             # removed file
 
-            if path == ETC_FIREWALLD_SERVICES:
+            if path == config.ETC_FIREWALLD_SERVICES:
                 # removed custom service
                 for x in self._services.keys():
                     obj = self._services[x]
                     if obj.filename == filename:
                         del self._services[x]
-                        if obj.name in self._default_services:
-                            return ("update", self._default_services[obj.name])
+                        if obj.name in self._builtin_services:
+                            return ("update", self._builtin_services[obj.name])
                         return ("remove", obj)
             else:
                 # removed builtin service
-                for x in self._default_services.keys():
-                    obj = self._default_services[x]
+                for x in self._builtin_services.keys():
+                    obj = self._builtin_services[x]
                     if obj.filename == filename:
-                        del self._default_services[x]
+                        del self._builtin_services[x]
                         if obj.name not in self._services:
                             # update dbus service
                             return ("remove", obj)
@@ -421,24 +600,30 @@ class FirewallConfig(object):
 
         # new or updated file
 
-        obj = service_reader(filename, path)
+        log.debug1("Loading service file '%s'", name)
+        try:
+            obj = service_reader(filename, path)
+        except Exception as msg:
+            log.error("Failed to load service file '%s': %s", filename, msg)
+            return (None, None)
 
         # new service
-        if obj.name not in self._default_services and obj.name not in self._services:
+        if obj.name not in self._builtin_services and obj.name not in self._services:
             self.add_service(obj)
             return ("new", obj)
 
         # updated service
-        if path == ETC_FIREWALLD_SERVICES:
+        if path == config.ETC_FIREWALLD_SERVICES:
             # custom service update
             if obj.name in self._services:
+                obj.default = self._services[obj.name].default
                 self._services[obj.name] = obj
             return ("update", obj)
         else:
-            if obj.name in self._default_services:
+            if obj.name in self._builtin_services:
                 # builtin service update
-                del self._default_services[obj.name]
-                self._default_services[obj.name] = obj
+                del self._builtin_services[obj.name]
+                self._builtin_services[obj.name] = obj
 
                 if obj.name not in self._services:
                     # update dbus service
@@ -452,22 +637,18 @@ class FirewallConfig(object):
 
     def _remove_service(self, obj):
         if obj.name not in self._services:
-            raise FirewallError(INVALID_SERVICE,
+            raise FirewallError(errors.INVALID_SERVICE,
                                 "'%s' not in self._services" % obj.name)
-        if obj.path != ETC_FIREWALLD_SERVICES:
-            raise FirewallError(INVALID_DIRECTORY,
-                        "'%s' != '%s'" % (obj.path, ETC_FIREWALLD_SERVICES))
+        if obj.path != config.ETC_FIREWALLD_SERVICES:
+            raise FirewallError(errors.INVALID_DIRECTORY,
+                                "'%s' != '%s'" % \
+                                (obj.path, config.ETC_FIREWALLD_SERVICES))
         os.remove("%s/%s.xml" % (obj.path, obj.name))
         del self._services[obj.name]
 
-    def is_builtin_service(self, obj):
-        if obj.default or obj.name in self._default_services:
-            return True
-        return False
-
     def check_builtin_service(self, obj):
-        if self.is_builtin_service(obj):
-            raise FirewallError(BUILTIN_SERVICE,
+        if obj.builtin or not obj.default:
+            raise FirewallError(errors.BUILTIN_SERVICE,
                                 "'%s' is built-in service" % obj.name)
 
     def remove_service(self, obj):
@@ -487,81 +668,78 @@ class FirewallConfig(object):
 
     def get_zones(self):
         return sorted(set(list(self._zones.keys()) + \
-                          list(self._default_zones.keys())))
+                          list(self._builtin_zones.keys())))
 
     def add_zone(self, obj):
-        if obj.default:
-            self._default_zones[obj.name] = obj
+        if obj.builtin:
+            self._builtin_zones[obj.name] = obj
         else:
             self._zones[obj.name] = obj
 
     def forget_zone(self, name):
-        if name in self._default_zones:
-            del self._default_zones[name]
+        if name in self._builtin_zones:
+            del self._builtin_zones[name]
         if name in self._zones:
             del self._zones[name]
 
     def get_zone(self, name):
         if name in self._zones:
             return self._zones[name]
-        elif name in self._default_zones:
-            return self._default_zones[name]
-        raise FirewallError(INVALID_ZONE, "get_zone(): %s" % name)
-
-    def zone_has_defaults(self, name):
-        return (name in self._zones and name in self._default_zones)
-
-    def zone_is_default(self, name):
-        return name in self._default_zones
+        elif name in self._builtin_zones:
+            return self._builtin_zones[name]
+        raise FirewallError(errors.INVALID_ZONE, "get_zone(): %s" % name)
 
     def load_zone_defaults(self, obj):
         if obj.name not in self._zones:
-            raise FirewallError(NO_DEFAULTS,
-                                "'%s' not in self._zones" % obj.name)
+            raise FirewallError(errors.NO_DEFAULTS, obj.name)
         elif self._zones[obj.name] != obj:
-            raise FirewallError(NO_DEFAULTS,
+            raise FirewallError(errors.NO_DEFAULTS,
                                 "self._zones[%s] != obj" % obj.name)
-        elif obj.name not in self._default_zones:
-            raise FirewallError(NO_DEFAULTS,
-                                "'%s' not in self._default_zones" % obj.name)
+        elif obj.name not in self._builtin_zones:
+            raise FirewallError(errors.NO_DEFAULTS,
+                                "'%s' not in self._builtin_zones" % obj.name)
         self._remove_zone(obj)
-        return self._default_zones[obj.name]
+        return self._builtin_zones[obj.name]
 
     def get_zone_config(self, obj):
         return obj.export_config()
 
-    def set_zone_config(self, obj, config):
-        if obj.default:
+    def set_zone_config(self, obj, conf):
+        if obj.builtin:
             x = copy.copy(obj)
             x.fw_config = self
-            x.import_config(config)
-            x.path = ETC_FIREWALLD_ZONES
-            x.default = False
+            x.import_config(conf)
+            x.path = config.ETC_FIREWALLD_ZONES
+            x.builtin = False
+            if obj.path != x.path:
+                x.default = False
             self.add_zone(x)
             zone_writer(x)
             return x
         else:
             obj.fw_config = self
-            obj.import_config(config)
+            obj.import_config(conf)
             zone_writer(obj)
             return obj
 
-    def new_zone(self, name, config):
+    def new_zone(self, name, conf):
         try:
             self.get_zone(name)
         except:
             pass
         else:
-            raise FirewallError(NAME_CONFLICT, "new_zone(): '%s'" % name)
+            raise FirewallError(errors.NAME_CONFLICT, "new_zone(): '%s'" % name)
 
         x = Zone()
         x.check_name(name)
         x.fw_config = self
-        x.import_config(config)
+        x.import_config(conf)
         x.name = name
         x.filename = "%s.xml" % name
-        x.path = ETC_FIREWALLD_ZONES
-        x.default = False
+        x.path = config.ETC_FIREWALLD_ZONES
+        # It is not possible to add a new one with a name of a buitin
+        x.builtin = False
+        x.default = True
 
         zone_writer(x)
         self.add_zone(x)
@@ -574,21 +752,21 @@ class FirewallConfig(object):
         if not os.path.exists(name):
             # removed file
 
-            if path == ETC_FIREWALLD_ZONES:
+            if path.startswith(config.ETC_FIREWALLD_ZONES):
                 # removed custom zone
                 for x in self._zones.keys():
                     obj = self._zones[x]
                     if obj.filename == filename:
                         del self._zones[x]
-                        if obj.name in self._default_zones:
-                            return ("update", self._default_zones[obj.name])
+                        if obj.name in self._builtin_zones:
+                            return ("update", self._builtin_zones[obj.name])
                         return ("remove", obj)
             else:
                 # removed builtin zone
-                for x in self._default_zones.keys():
-                    obj = self._default_zones[x]
+                for x in self._builtin_zones.keys():
+                    obj = self._builtin_zones[x]
                     if obj.filename == filename:
-                        del self._default_zones[x]
+                        del self._builtin_zones[x]
                         if obj.name not in self._zones:
                             # update dbus zone
                             return ("remove", obj)
@@ -601,25 +779,38 @@ class FirewallConfig(object):
 
         # new or updated file
 
-        obj = zone_reader(filename, path)
+        log.debug1("Loading zone file '%s'", name)
+        try:
+            obj = zone_reader(filename, path)
+        except Exception as msg:
+            log.error("Failed to load zone file '%s': %s", filename, msg)
+            return (None, None)
+
         obj.fw_config = self
 
+        if path.startswith(config.ETC_FIREWALLD_ZONES) and \
+           len(path) > len(config.ETC_FIREWALLD_ZONES):
+            # custom combined zone part
+            obj.name = "%s/%s" % (os.path.basename(path),
+                                  os.path.basename(filename)[0:-4])
+
         # new zone
-        if obj.name not in self._default_zones and obj.name not in self._zones:
+        if obj.name not in self._builtin_zones and obj.name not in self._zones:
             self.add_zone(obj)
             return ("new", obj)
 
         # updated zone
-        if path == ETC_FIREWALLD_ZONES:
+        if path.startswith(config.ETC_FIREWALLD_ZONES):
             # custom zone update
             if obj.name in self._zones:
+                obj.default = self._zones[obj.name].default
                 self._zones[obj.name] = obj
             return ("update", obj)
         else:
-            if obj.name in self._default_zones:
+            if obj.name in self._builtin_zones:
                 # builtin zone update
-                del self._default_zones[obj.name]
-                self._default_zones[obj.name] = obj
+                del self._builtin_zones[obj.name]
+                self._builtin_zones[obj.name] = obj
 
                 if obj.name not in self._zones:
                     # update dbus zone
@@ -633,22 +824,19 @@ class FirewallConfig(object):
 
     def _remove_zone(self, obj):
         if obj.name not in self._zones:
-            raise FirewallError(INVALID_ZONE,
+            raise FirewallError(errors.INVALID_ZONE,
                                 "'%s' not in self._zones" % obj.name)
-        if not obj.path.startswith(ETC_FIREWALLD_ZONES):
-            raise FirewallError(INVALID_DIRECTORY,
-                "'%s' doesn't start with '%s'" % (obj.path, ETC_FIREWALLD_ZONES))
-        os.remove("%s/%s.xml" % (ETC_FIREWALLD_ZONES, obj.name))
+        if not obj.path.startswith(config.ETC_FIREWALLD_ZONES):
+            raise FirewallError(errors.INVALID_DIRECTORY,
+                                "'%s' doesn't start with '%s'" % \
+                                (obj.path, config.ETC_FIREWALLD_ZONES))
+        os.remove("%s/%s.xml" % (config.ETC_FIREWALLD_ZONES, obj.name))
         del self._zones[obj.name]
 
-    def is_builtin_zone(self, obj):
-        if obj.default or obj.name in self._default_zones:
-            return True
-        return False
-
     def check_builtin_zone(self, obj):
-        if self.is_builtin_zone(obj):
-            raise FirewallError(BUILTIN_ZONE, "'%s' is built-in zone" % obj.name)
+        if obj.builtin or not obj.default:
+            raise FirewallError(errors.BUILTIN_ZONE,
+                                "'%s' is built-in zone" % obj.name)
 
     def remove_zone(self, obj):
         self.check_builtin_zone(obj)
