@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2010-2012 Red Hat, Inc.
+# Copyright (C) 2010-2016 Red Hat, Inc.
 #
 # Authors:
 # Thomas Woerner <twoerner@redhat.com>
@@ -19,14 +19,16 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
+__all__ = [ "FirewallDirect" ]
+
 import os.path
-from firewall.config import *
 from firewall import functions
 from firewall.fw_types import *
 from firewall.core import ipXtables
 from firewall.core import ebtables
 from firewall.core.logger import log
-from firewall.errors import *
+from firewall import errors
+from firewall.errors import FirewallError
 
 ############################################################################
 #
@@ -54,13 +56,15 @@ class FirewallDirect(object):
         self.__init_vars()
 
     def set_permanent_config(self, obj):
+        self._obj = obj
+
+    def apply_direct(self):
         # Apply permanent configuration and save the obj to be able to
         # remove permanent configuration settings within get_runtime_config
         # for use in firewalld reload.
-        self._obj = obj
-        self.set_config((obj.get_all_chains(),
-                         obj.get_all_rules(),
-                         obj.get_all_passthroughs()))
+        self.set_config((self._obj.get_all_chains(),
+                         self._obj.get_all_rules(),
+                         self._obj.get_all_passthroughs()))
 
     def get_runtime_config(self):
         # Return only runtime changes
@@ -96,8 +100,8 @@ class FirewallDirect(object):
     def get_config(self):
         return (self._chains, self._rules, self._passthroughs)
 
-    def set_config(self, config):
-        (_chains, _rules, _passthroughs) = config
+    def set_config(self, conf):
+        (_chains, _rules, _passthroughs) = conf
         for table_id in _chains:
             (ipv, table) = table_id
             for chain in _chains[table_id]:
@@ -127,7 +131,7 @@ class FirewallDirect(object):
     def _check_ipv(self, ipv):
         ipvs = ['ipv4', 'ipv6', 'eb']
         if ipv not in ipvs:
-            raise FirewallError(INVALID_IPV,
+            raise FirewallError(errors.INVALID_IPV,
                                 "'%s' not in '%s'" % (ipv, ipvs))
 
     def _check_ipv_table(self, ipv, table):
@@ -136,7 +140,7 @@ class FirewallDirect(object):
         tables = ipXtables.BUILT_IN_CHAINS.keys() if ipv in [ 'ipv4', 'ipv6' ] \
                                          else ebtables.BUILT_IN_CHAINS.keys()
         if table not in tables:
-            raise FirewallError(INVALID_TABLE,
+            raise FirewallError(errors.INVALID_TABLE,
                                 "'%s' not in '%s'" % (table, tables))
 
     def _check_builtin_chain(self, ipv, table, chain):
@@ -147,14 +151,14 @@ class FirewallDirect(object):
             built_in_chains = ebtables.BUILT_IN_CHAINS[table]
             our_chains = ebtables.OUR_CHAINS[table]
         if chain in built_in_chains:
-            raise FirewallError(BUILTIN_CHAIN,
+            raise FirewallError(errors.BUILTIN_CHAIN,
                  "chain '%s' is built-in chain" % chain)
         if chain in our_chains:
-            raise FirewallError(BUILTIN_CHAIN,
+            raise FirewallError(errors.BUILTIN_CHAIN,
                  "chain '%s' is reserved" % chain)
         if ipv in [ "ipv4", "ipv6" ]:
             if self._fw.zone.zone_from_chain(ipv, table, chain) != None:
-                raise FirewallError(INVALID_CHAIN,
+                raise FirewallError(errors.INVALID_CHAIN,
                                     "Chain '%s' is reserved" % chain)
 
     # DIRECT CHAIN
@@ -167,12 +171,12 @@ class FirewallDirect(object):
         if add:
             if table_id in self._chains and \
                     chain in self._chains[table_id]:
-                raise FirewallError(ALREADY_ENABLED,
+                raise FirewallError(errors.ALREADY_ENABLED,
                      "chain '%s' already is in '%s:%s'" % (chain, ipv, table))
         else:
             if table_id not in self._chains or \
                     chain not in self._chains[table_id]:
-                raise FirewallError(NOT_ENABLED,
+                raise FirewallError(errors.NOT_ENABLED,
                     "chain '%s' is not in '%s:%s'" % (chain, ipv, table))
 
         rule = [ "-t", table ]
@@ -188,7 +192,7 @@ class FirewallDirect(object):
             self._fw.rule(ipv, rule)
         except Exception as msg:
             log.debug2(msg)
-            raise FirewallError(COMMAND_FAILED, msg)
+            raise FirewallError(errors.COMMAND_FAILED, msg)
 
         if add:
             self._chains.setdefault(table_id, [ ]).append(chain)
@@ -249,13 +253,13 @@ class FirewallDirect(object):
         if enable:
             if chain_id in self._rules and \
                     rule_id in self._rules[chain_id]:
-                raise FirewallError(ALREADY_ENABLED,
+                raise FirewallError(errors.ALREADY_ENABLED,
                                     "rule '%s' already is in '%s:%s:%s'" % \
                                     (args, ipv, table, chain))
         else:
             if chain_id not in self._rules or \
                     rule_id not in self._rules[chain_id]:
-                raise FirewallError(NOT_ENABLED,
+                raise FirewallError(errors.NOT_ENABLED,
                                     "rule '%s' is not in '%s:%s:%s'" % \
                                     (args, ipv, table, chain))
 
@@ -319,7 +323,7 @@ class FirewallDirect(object):
             self._fw.rule(ipv, rule)
         except Exception as msg:
             log.debug2(msg)
-            raise FirewallError(COMMAND_FAILED, msg)
+            raise FirewallError(errors.COMMAND_FAILED, msg)
 
         if enable:
             if chain_id not in self._rules:
@@ -372,7 +376,7 @@ class FirewallDirect(object):
             return self._fw.rule(ipv, args)
         except Exception as msg:
             log.debug2(msg)
-            raise FirewallError(COMMAND_FAILED, msg)
+            raise FirewallError(errors.COMMAND_FAILED, msg)
 
     # DIRECT PASSTHROUGH (tracked)
 
@@ -382,12 +386,12 @@ class FirewallDirect(object):
         passthrough_id = (ipv, args)
         if enable:
             if ipv in self._passthroughs and args in self._passthroughs[ipv]:
-                raise FirewallError(ALREADY_ENABLED,
+                raise FirewallError(errors.ALREADY_ENABLED,
                                     "passthrough '%s', '%s'" % (ipv, args))
         else:
             if ipv not in self._passthroughs or \
                args not in self._passthroughs[ipv]:
-                raise FirewallError(NOT_ENABLED,
+                raise FirewallError(errors.NOT_ENABLED,
                                     "passthrough '%s', '%s'" % (ipv, args))
 
         if enable:
@@ -423,7 +427,7 @@ class FirewallDirect(object):
             self._fw.rule(ipv, _args)
         except Exception as msg:
             log.debug2(msg)
-            raise FirewallError(COMMAND_FAILED, msg)
+            raise FirewallError(errors.COMMAND_FAILED, msg)
 
         if enable:
             if ipv not in self._passthroughs:
@@ -476,7 +480,7 @@ class FirewallDirect(object):
         # intersection of args and not_allowed is not empty, i.e.
         # something from args is not allowed
         if len(args & not_allowed) > 0:
-            raise FirewallError(INVALID_PASSTHROUGH,
+            raise FirewallError(errors.INVALID_PASSTHROUGH,
                                 "arg '%s' is not allowed" %
                                 list(args & not_allowed)[0])
 
@@ -487,7 +491,7 @@ class FirewallDirect(object):
         # empty intersection of args and needed, i.e.
         # none from args contains any needed command
         if len(args & needed) == 0:
-            raise FirewallError(INVALID_PASSTHROUGH,
+            raise FirewallError(errors.INVALID_PASSTHROUGH,
                                 "no '-A', '-I' or '-N' arg")
 
     def reverse_passthrough(self, args):
@@ -527,4 +531,5 @@ class FirewallDirect(object):
             ret_args[idx] = replace_args[x]
             return ret_args
 
-        raise FirewallError(INVALID_PASSTHROUGH, "no '-A', '-I' or '-N' arg")
+        raise FirewallError(errors.INVALID_PASSTHROUGH,
+                            "no '-A', '-I' or '-N' arg")
