@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2011-2012 Red Hat, Inc.
+# Copyright (C) 2011-2016 Red Hat, Inc.
 #
 # Authors:
 # Thomas Woerner <twoerner@redhat.com>
@@ -82,7 +82,7 @@ class IcmpType(IO_Object):
 
 class icmptype_ContentHandler(IO_Object_ContentHandler):
     def startElement(self, name, attrs):
-        IO_Object_ContentHandler.startElement(self, name)
+        IO_Object_ContentHandler.startElement(self, name, attrs)
         self.item.parser_check_element_attrs(name, attrs)
 
         if name == "icmptype":
@@ -110,7 +110,8 @@ def icmptype_reader(filename, path):
     icmptype.check_name(icmptype.name)
     icmptype.filename = filename
     icmptype.path = path
-    icmptype.default = False if path.startswith(ETC_FIREWALLD) else True
+    icmptype.builtin = False if path.startswith(ETC_FIREWALLD) else True
+    icmptype.default = icmptype.builtin
     handler = icmptype_ContentHandler(icmptype)
     parser = sax.make_parser()
     parser.setContentHandler(handler)
