@@ -24,7 +24,9 @@ import os.path
 import shlex, pipes
 import string
 import sys
+import tempfile
 from firewall.core.logger import log
+from firewall.config import FIREWALLD_TEMPDIR, FIREWALLD_PIDFILE
 
 PY2 = sys.version < '3'
 
@@ -58,13 +60,28 @@ def getPortRange(ports):
     @return Array containing start and end port id for a valid range or -1 if port can not be found and -2 if port is too big for integer input or -1 for invalid ranges or None if the range is ambiguous.
     """
 
-    if isinstance(ports, int):
+    # "<port-id>" case
+    if isinstance(ports, int) or ports.isdigit():
         id = getPortID(ports)
         if id >= 0:
             return (id,)
         return id
 
     splits = ports.split("-")
+
+    # "<port-id>-<port-id>" case
+    if len(splits) == 2 and splits[0].isdigit() and splits[1].isdigit():
+        id1 = getPortID(splits[0])
+        id2 = getPortID(splits[1])
+        if id1 >= 0 and id2 >= 0:
+            if id1 < id2:
+                return (id1, id2)
+            elif id1 > id2:
+                return (id2, id1)
+            else: # ids are the same
+                return (id1,)
+
+    # everything else "<port-str>[-<port-str>]"
     matched = [ ]
     for i in range(len(splits), 0, -1):
         id1 = getPortID("-".join(splits[:i]))
@@ -222,17 +239,27 @@ def checkInterface(iface):
     #    return False
     return True
 
+def checkUINT32(val):
+    try:
+        x = int(val, 0)
+    except:
+        return False
+    else:
+        if x >= 0 and x <= 4294967295:
+            return True
+    return False
+
 def firewalld_is_active():
     """ Check if firewalld is active
 
     @return True if there is a firewalld pid file and the pid is used by firewalld
     """
 
-    if not os.path.exists("/var/run/firewalld.pid"):
+    if not os.path.exists(FIREWALLD_PIDFILE):
         return False
 
     try:
-        with open("/var/run/firewalld.pid", "r") as fd:
+        with open(FIREWALLD_PIDFILE, "r") as fd:
             pid = fd.readline()
     except:
         return False
@@ -251,14 +278,26 @@ def firewalld_is_active():
 
     return False
 
+def tempFile():
+    try:
+        if not os.path.exists(FIREWALLD_TEMPDIR):
+            os.mkdir(FIREWALLD_TEMPDIR, 0o750)
+
+        return tempfile.NamedTemporaryFile(mode='wt', prefix="temp.",
+                                           dir=FIREWALLD_TEMPDIR, delete=False)
+    except Exception as msg:
+        log.error("Failed to create temporary file: %s" % msg)
+        raise
+    return None
+
 def readfile(filename):
+    i = 1
     try:
         with open(filename, "r") as f:
-            line = "".join(f.readlines())
+            return f.readlines()
     except Exception as e:
         log.error('Failed to read file "%s": %s' % (filename, e))
-        return None
-    return line
+    return None
 
 def writefile(filename, line):
     try:
@@ -307,6 +346,18 @@ def check_single_address(ipv, source):
     else:
         return False
 
+def check_mac(mac):
+    if len(mac) == 12+5:
+        # 0 1 : 3 4 : 6 7 : 9 10 : 12 13 : 15 16
+        for i in (2, 5, 8, 11, 14):
+            if mac[i] != ":":
+                return False
+        for i in (0, 1, 3, 4, 6, 7, 9, 10, 12, 13, 15, 16):
+            if mac[i] not in string.hexdigits:
+                return False
+        return True
+    return False
+
 def uniqify(input):
     # removes duplicates from list, whilst preserving order
     output = []
